import random
import string
from requests.adapters import HTTPAdapter
from urllib3.poolmanager import PoolManager
from urllib3.util import ssl_
import ssl

CIPHERS = (
    'ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-SHA384:'
    'ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-SHA256:AES256-SHA'
)

conf = '''
[Interface]
PrivateKey = {private_key}
Address = {address}
DNS = 1.1.1.1
[Peer]
PublicKey = {public_key}
Endpoint = {endpoint}
AllowedIPs = 0.0.0.0/0'''


class TlsAdapter(HTTPAdapter):

    def __init__(self, ssl_options=0, **kwargs):
        self.ssl_options = ssl_options
        super(TlsAdapter, self).__init__(**kwargs)

    def init_poolmanager(self, *pool_args, **pool_kwargs):
        ctx = ssl_.create_urllib3_context(ciphers=CIPHERS, cert_reqs=ssl.CERT_REQUIRED, options=self.ssl_options)
        self.poolmanager = PoolManager(*pool_args,
                                       ssl_context=ctx,
                                       **pool_kwargs)


class Colored:
    HEADER = '\033[43;40m'
    PINK = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    RED = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    YELLOW = '\033[33;1m'


def generate_string(length):
    letters = string.ascii_letters + string.digits
    return ''.join(random.choice(letters) for _ in range(length))



import datetime
import os
from base64 import b64encode

import requests
from nacl.bindings import crypto_scalarmult_base

from warpy.utils import generate_string, conf, TlsAdapter

api_version = "v0a884"

session = requests.session()
session.mount("https://", TlsAdapter())


class WarpPlus:

    @staticmethod
    def generate_key():
        private_key = os.urandom(32)
        public_key = crypto_scalarmult_base(private_key)
        return b64encode(private_key).decode('utf-8'), b64encode(public_key).decode('utf-8')

    def enable_warp(self, config):
        data = {"warp_enabled": True}
        url = 'https://api.cloudflareclient.com/' + api_version + '/reg/' + config['id']
        headers = {"Accept-Encoding": "gzip",
                   "User-Agent": "okhttp/3.12.1",
                   "Authorization": "Bearer {}".format(config['token']),
                   "Content-Type": "application/json; charset=UTF-8"}
        req = session.request("PATCH", url, json=data, headers=headers)
        req.raise_for_status()
        req = req.json()
        assert req["warp_enabled"] is True

    def register(self, key=None, referrer=None):

        url = 'https://api.cloudflareclient.com/' + api_version + '/reg'

        headers = {"Accept-Encoding": "gzip",
                   "User-Agent": "okhttp/3.12.1",
                   "Content-Type": "application/json; charset=UTF-8"}

        install_id = generate_string(11)
        key = key if key else self.generate_key()
        data = {"key": key[1],
                "install_id": "",
                "fcm_token": "",
                "referrer": referrer or "",
                "warp_enabled": True,
                "tos": datetime.datetime.now().isoformat()[:-7] + "Z",
                "model": "",
                "type": "Android",
                "locale": "en_US"}
        req = session.request('POST', url, headers=headers, json=data)
        if req.status_code != 200:
            return {}
        req_json = dict(req.json())
        req_json['key'] = {"public_key": req_json['config']['peers'][0]['public_key'], "private_key": key[0]}
        return req_json

    @staticmethod
    def get_info(ID, token):
        url = 'https://api.cloudflareclient.com/' + api_version + '/reg/' + ID.strip()
        headers = {"Authorization": "Bearer " + token.strip()}
        req = session.request("GET", url, headers=headers)
        if req.status_code != 200:
            raise Exception(req.text)
        return req.json()

    @staticmethod
    def export_to_wireguard(config):
        conf_text = conf.format(private_key=config['key']['private_key'],
                                public_key=config['config']['peers'][0]['public_key'],
                                address=config['config']['interface']['addresses']['v4'],
                                endpoint=config['config']['peers'][0]['endpoint']['host'])
        return conf_text

    def increase_quota(self, config):
        return self.register(referrer=config['id'])



#! /usr/bin/python3

import sys
from time import sleep

from pathlib import Path
import os
import re
import json

from warpy.cli import WarpPlus
from warpy.utils import Colored

warp_plus = WarpPlus()

home = str(Path.home())
config_dir = os.path.join(home, ".warpy")


def increase_user_quota(config, user):
    counter = 0
    _input = input("({}{}{}) Enter quota to increase: ".format(Colored.OKGREEN, user, Colored.ENDC))
    while not _input.isdigit() or int(_input) < 1:
        _input = input("({}{}{}) ({}Invalid{}) Enter quota to increase: ".format(Colored.OKGREEN, user, Colored.ENDC,
                                                                                 Colored.RED, Colored.ENDC))
    print("\nWait until the end, Do not close the script!")
    quota = int(_input)
    while counter != quota:
        increase = warp_plus.increase_quota(config)
        if not increase:
            print("60 seconds sleep..., (Do not close the script)")
            sleep(60)
        elif counter != quota and counter % 3 == 0 and counter != 0:
            print("60 seconds sleep..., (Do not close the script)")
            sleep(60)
        print("+1GB Added")
        counter += 1
        sleep(1)
    print("Done!")


def get_config(file_name):
    file_path = os.path.join(config_dir, file_name)
    file = open(file_path, 'r')
    conf = json.loads(file.read())
    file.close()
    return conf


def press_any_key():
    print()
    print(Colored.PINK + "press any key to continue..." + Colored.ENDC)
    sys.stdin.read(1)


def show_user_info(config):
    info = warp_plus.get_info(config["id"], config["token"])
    text = '\nID: {}\n'.format(config['id'])
    text += "Public Key: {}\n".format(config['key']['public_key'])
    text += "Account Type: {}\n".format(info['account']['account_type'])
    text += "Quota: {}GB".format(round(int(info['account']['quota']) / (10 ** 9), 2))
    print(text)


def user_details(file_name):
    _input = None
    while _input is not None or _input != 0:
        print(Colored.YELLOW + "1. Show WireGuard Config" + Colored.ENDC)
        print(Colored.YELLOW + "2. Show User Info" + Colored.ENDC)
        print(Colored.YELLOW + "3. Increase Quota" + Colored.ENDC)
        print(Colored.ENDC + "0. Back To Main Menu" + Colored.ENDC)
        print()
        user = file_name.replace('.json', '')
        conf = get_config(file_name)
        _input = input("({}{}{}) Please enter a number: ".format(Colored.OKGREEN, user, Colored.ENDC))
        while not _input.isdigit():
            _input = input("({}{}{}) ({}Invalid{}) Please enter a number: ".format(Colored.OKGREEN, user, Colored.ENDC,
                                                                                   Colored.RED, Colored.ENDC))
        _input = int(_input)
        if _input == 1:
            print(warp_plus.export_to_wireguard(conf))
            press_any_key()
        elif _input == 2:
            show_user_info(conf)
            press_any_key()
        elif _input == 3:
            increase_user_quota(conf, user)
            press_any_key()
        elif _input == 0:
            break


def show_users(remove=False):
    dir_files = [i for i in os.listdir(config_dir) if i.endswith(".json")]
    print("\n{}List of Users:{}".format(Colored.YELLOW, Colored.ENDC))
    if not dir_files:
        print("Empty\n")
        return
    counter = 1
    for i in dir_files:
        print(str(counter) + ". " + Colored.OKGREEN + i[:-5] + Colored.ENDC)
        counter += 1
    text = "Enter user number (0 to back): " if not remove else "Enter user number to " \
                                                                "{}remove{} (0 to back): ".format(Colored.RED,
                                                                                                  Colored.ENDC)
    _input = input("\n" + text)
    while not _input.isdigit() or int(_input) > len(dir_files) or int(_input) < 0:
        _input = input("({}Invalid{}) {}: ".format(Colored.RED, Colored.ENDC, text))
    _input = int(_input)
    if _input == 0:
        return
    _input -= 1
    if remove:
        return dir_files[_input]
    print()
    return user_details(dir_files[_input])


def make_filename(file_name):
    file_name = file_name.replace(" ", "_")
    file_name = file_name.replace(".", "_")
    file_name = re.sub(r"[^a-z0-9_]", "", file_name.lower().strip())
    return file_name


def save_config(file_name, config):
    file_name = file_name + ".json"
    if not os.path.isdir(config_dir):
        os.mkdir(config_dir)
    file_path = os.path.join(config_dir, file_name)
    file = open(file_path, "w")
    file.write(json.dumps(config))
    file.close()


def print_header():
    print(Colored.PINK + Colored.HEADER + " ====================== " + Colored.ENDC)
    print(Colored.PINK + Colored.HEADER + " WARP+ WireGuard Python " + Colored.ENDC)
    print(Colored.PINK + Colored.HEADER + " By Arian Amiramjadi    " + Colored.ENDC)
    print(Colored.PINK + Colored.HEADER + " ====================== " + Colored.ENDC)
    print()


def print_options():
    print(Colored.YELLOW + "1. Register User" + Colored.ENDC)
    print(Colored.YELLOW + "2. Show Users" + Colored.ENDC)
    print(Colored.RED + "3. Remove User" + Colored.ENDC)
    print(Colored.ENDC + "0. Exit Script" + Colored.ENDC)
    print()


def show_menu():
    print_header()
    print_options()


def register():
    user = input("Please specify a username: ")
    user = make_filename(user)
    while len(user) == 0:
        user = input("{}(Invalid){} Please specify a username: ".format(Colored.RED, Colored.ENDC))
        user = make_filename(user)
    while os.path.isfile(os.path.join(str(Path.home()), ".warpy", user + '.json')):
        user = input("({}UserExists{}) Please specify a username: ".format(Colored.RED, Colored.ENDC))
        user = make_filename(user)
    reg = warp_plus.register(warp_plus.generate_key())
    save_config(user, reg)
    print(warp_plus.export_to_wireguard(reg))
    warp_plus.increase_quota(reg)


def main():
    invalid = False
    _input = None
    show_menu()
    while _input is not None or _input != 0:
        if invalid:
            text = "({}Invalid{}) Please enter a number: ".format(Colored.RED, Colored.ENDC)
        else:
            text = "Please enter a number: "
        _input = input(text)
        while not _input.isdigit():
            _input = input("({}Invalid{}) Please enter a number: ".format(Colored.RED, Colored.ENDC))
        _input = int(_input)
        invalid = False
        if _input == 0:
            exit()
        elif _input == 1:
            register()
            press_any_key()
        elif _input == 2:
            show_users()
        elif _input == 3:
            user = show_users(remove=True)
            if user:
                sure = input(Colored.RED + "Are you sure you want to remove this user (y/n default: No) ? " +
                             Colored.ENDC).lower()
                if sure == "y" or sure == "yes":
                    os.remove(os.path.join(config_dir, user))
                    print("User `{}` removed successfully.".format(Colored.OKGREEN + user.replace(".json", "") +
                                                                   Colored.ENDC))
                    press_any_key()
        else:
            invalid = True
        if not invalid:
            show_menu()


if __name__ == '__main__':
    main()



from setuptools import setup
from os import path

current_dir = path.abspath(path.dirname(__file__))

with open(path.join(current_dir, 'README.md')) as f:
    description = f.read()

setup(name='warpy',
      version='0.7.3',
      long_description=description,
      long_description_content_type='text/markdown',
      description='A simple cli to get WARP+ as WireGuard configuration',
      url='https://github.com/warp-plus/warpy',
      download_url='https://github.com/warp-plus/warpy/archive/v_07.tar.gz',
      keywords=['warpy', 'warp+', 'warp-plus', 'warp wireguard'],
      license='GPL',
      author='Arian Amiramjadi',
      author_email='me@arian.lol',
      packages=['warpy'],
      install_requires=['pynacl', "requests"],
      entry_points={'console_scripts': [
          'warpy = warpy.__main__:main',
      ]},
      classifiers=[
          'Development Status :: 3 - Alpha',
          # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
          'Intended Audience :: Developers',
          'Topic :: Software Development :: Build Tools',
          'License :: OSI Approved :: GNU General Public License (GPL)',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
      ],
      )
